/** @file   he-file.c   Funktionen fuer die Arbeit mit Dateien.
*/

/* Copyright (C) 2014 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#include <stdio.h>
#if HAVE_ERRNO_H
#include <errno.h>
#endif
#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include "he-error.h"
#include "he-file.h"



FILE *
hsm_et_file_open(const char *name, const char *mode, int *ec)
{
  FILE      *back   =   NULL;           /* Ergebnis der Funktion */
  int        errsav;                    /* Wert von errno zwischenspeichern */

  if ((NULL != name) && (NULL != mode)) {
#if HE_ON_WINDOWS
    errsav = fopen_s(&back, name, mode);
    if (0 != errsav) {
      back = NULL;
      fputs("FEHLER: Datei \"", stderr);
      fputs(name, stderr);
      fputs("\" nicht geoeffnet!\n", stderr);
      errno = errsav;
      perror("fopen");
      fflush(stderr);
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
#else
    back = fopen(name, mode);
    if (NULL == back) {
      errsav = errno;
      fputs("FEHLER: Datei \"", stderr);
      fputs(name, stderr);
      fputs("\" nicht geoeffnet!\n", stderr);
      errno = errsav;
      perror("fopen");
      fflush(stderr);
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
#endif
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return back;
}



int
hsm_et_file_close(FILE *fp, const char *name, int *ec)
{
  int        errsav;            /* Zwischenspeichern von errno */
  int        back   =   0;      /* Ergebnis der Funktion */

  if (NULL != fp) {
    if (0 == fclose(fp)) {
      back = 1;
    } else {
      errsav = errno;
      if (NULL != name) {
        fputs("FEHLER: Problem beim Schliessen der Datei \"", stderr);
        fputs(name, stderr);
        fputs("\"!\n", stderr);
      } else {
        fputs("FEHLER: Problem beim Schliessen der Datei!\n", stderr);
      }
      errno = errsav;
      perror("fclose");
      fflush(stderr);
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return back;
}



int
hsm_et_file_puts(FILE *fp, const char *src, const char *name, int *ec)
{
  int       errsav;                     /* Wert von errno zwischenspeichern */
  int       back    =   0;              /* Ergebnis der Funktion */

  if ((NULL != fp) && (NULL != src)) {
    if (EOF != fputs(src, fp)) {
      back = 1;
    } else {
      errsav = errno;
      if (NULL != name) {
        fputs("FEHLER: Schreiben von Daten in Datei \"", stderr);
        fputs(name, stderr);
        fputs("\" fehlgeschlagen!\n", stderr);
      } else {
        fputs("FEHLER: Daten konnten nicht geschrieben werden!\n", stderr);
      }
      errno = errsav;
      perror("fputs");
      fflush(stderr);
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return back;
}



int
hsm_et_file_size(uintmax_t  *szptr, const char *name, int *ec)
{
#if HE_ON_WINDOWS
  struct __stat64   stbuf;
  int               errsav;
  int               back = 0;
  if ((NULL != szptr) && (NULL != name)) {
    if (0 == _stat64(name, &stbuf)) {
      back = 1;
      *szptr = (uintmax_t)(stbuf.st_size);
    } else {
      *szptr = (uintmax_t)0UL;
      errsav = errno;
      fputs("FEHLER: Groessenbestimmung fehlgeschlagen fuer \"", stderr);
      fputs(name, stderr);
      fputs("\"!\n", stderr);
      errno = errsav;
      perror("stat64");
      fflush(stderr);
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return back;
#else
  struct stat   stbuf;
  int           back    =   0;
  int           errsav;
  if ((NULL != szptr) && (NULL != name)) {
    if (0 == stat(name, &stbuf)) {
      *szptr = (uintmax_t)(stbuf.st_size);
      back = 1;
    } else {
      *szptr = (uintmax_t)0;
      errsav = errno;
      fputs("FEHLER: Groessenbestimmung fehlgeschlagen fuer \"", stderr);
      fputs(name, stderr);
      fputs("\"!\n", stderr);
      errno = errsav;
      perror("stat");
      fflush(stderr);
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return    back;
#endif
}



int
hsm_et_file_pointer_size(uintmax_t *szptr, FILE *fp, const char *name, int *ec)
{
#if HE_ON_WINDOWS
  struct __stat64   stbuf;
  int               back    = 0;
  int               errsav;
  if ((NULL != szptr) && (NULL != fp)) {
    if (0 == _fstat64(_fileno(fp), &stbuf)) {
      *szptr = (uintmax_t)(stbuf.st_size);
      back = 1;
    } else {
      *szptr = (uintmax_t)0UL;
      if (NULL != name) {
        errsav = errno;
        fputs("FEHLER: Groessenbestimmung fuer Datei \"", stderr);
        fputs(name, stderr);
        fputs("\" fehlgeschlagen!\n", stderr);
        errno = errsav;
        perror("_fstat64");
        fflush(stderr);
      }
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return back;
#else
  struct stat   stbuf;
  int           back    = 0;
  int           errsav;
  if ((NULL != szptr) && (NULL != fp)) {
    if (0 == fstat(fileno(fp), &stbuf)) {
      *szptr = (uintmax_t)(stbuf.st_size);
      back = 1;
    } else {
      *szptr = (uintmax_t)0UL;
      if (NULL != name) {
        errsav = errno;
        fputs("FEHLER: Groessenbestimmung fuer Datei \"", stderr);
        fputs(name, stderr);
        fputs("\" fehlgeschlagen!\n", stderr);
        errno = errsav;
        perror("_fstat64");
        fflush(stderr);
      }
      hsm_et_error_set(ec, HE_ERROR_IO);
    }
  } else {
    hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
  }
  return back;
#endif
}

/* vim: set ai sw=4 ts=4 expandtab : */
