/** @file   ex057.c Beispielprogramm fuer Zeiger und Felder von Strukturen.
*/

/* Copyright (C) 2014-2016 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>

#include "he-svg.h"


/**	Ein geometrischer Punkt.
*/

typedef struct {
  double	x;	/**< X-Koordinate. */
  double	y;	/**< Y-Koordinate. */
} PUNKT;


/**	Eine Linie.
*/

typedef struct {
  PUNKT	P0;		/**< Anfangspunkt. */
  PUNKT	P1;		/**< Endpunkt. */
} LINIE;


/**	Rechteck.
*/

typedef struct {
  double	xmin;	/**< Linker x-Wert. */
  double	xmax;	/**< Rechter x-Wert. */
  double	ymin;	/**< Unterer y-Wert. */
  double	ymax;	/**< Oberer y-Wert. */
} RECHTECK;


/**	Konstante Variable vom Typ Rechteck.
*/

static const RECHTECK test_re = { 0.0, 10.0, 0.0, 12.0 };


/**	Feld mit mehreren Variablen vom Typ Linie.
	Jede Linie besteht ihrerseits aus 2 Punkten.
	Jeder dieser Punkte wiederum besteht aus x- und y-Wert.
*/

static const LINIE linien[] = {
  {{  1.0,  1.0 }, {  9.0,  9.0 }}, /* Erste Linie, bestehend aus P0 und P1 */
  {{  2.0, 11.0 }, {  7.0,  8.0 }}, /* Naechste Linie ... */
  {{  4.0,  3.0 }, {  9.5,  9.3 }},
  {{ -1.0,  2.0 }, { 10.5, 10.5 }},
  {{  8.5,  8.5 }, {  2.0,  2.5 }},
  {{  5.5,  6.5 }, {  7.5,  8.3 }},
  {{  3.0, 13.0 }, {  7.0, 13.5 }},
  {{  1.0,  9.8 }, {  8.0,  1.0 }},
  {{ -3.0,  4.2 }, { 13.0,  4.4 }},
  {{  1.0,  2.0 }, {  3.0,  4.0 }}
};


/**	Anzahl der Linien im Feld linien.
	Zur Ermittelung der Anzahl wird die Gesamtgroesse der
	Variable linien durch die Groesse des Typs dividiert.
*/

static const size_t anzahl_linien = sizeof(linien)/sizeof(LINIE);


/** Name der zu erzeugenden SVG-Datei.
*/

const char dn_erg[] = {
#ifdef _WIN32
"C:\\Temp\\C\\ex075.svg"
#else
"ex075.svg"
#endif
};


/** SVG-Datei erzeugen.
*/
static
void
svg_ausgabe(void);


/**	Pruefe im Punkt im Rechteck oder auf dessen Rand liegt.
	@param	p	Zu pruefender Punkt.
	@param	r	Rechteck, in oder auf dessen Rand der Punkt liegen soll.
	@return	1 wenn der Punkt im Rechteck liegt, ansonsten 0.
*/

static
int
punkt_im_rechteck(const PUNKT *p, const RECHTECK *r)
{
  int	res	= 0;
  if((p->x >= r->xmin) && (p->x <= r->xmax) && (p->y >= r->ymin) && (p->y <= r->ymax)) {
    res = 1;
  }
  return res;
}


/**	Pruefe, ob Linie vollstaendig im Rechteck oder auf
	dessen Rand liegt (also
	keinen Punkt ausserhalb des Rechtecks hat).
	@param	p	Zu pruefende Linie.
	@param	r	Rechteck, in dem die Linie liegen soll.
	@return	1 wenn Linie im Rechteck liegt, ansonsten 0.
*/

static
int
linie_im_rechteck(const LINIE *p, const RECHTECK *r)
{
  int		res	= 0;
  if(punkt_im_rechteck(&(p->P0), r)) {
    if(punkt_im_rechteck(&(p->P1), r)) {
      res = 1;
    }
  }
  return res;
}


/** Hauptprogramm.
    @return 0 bei Erfolg.
*/

int main(void)
{
  LINIE const	*pl		= NULL;	/* Zeiger auf aktuelle Linie */
  size_t	 i;			/* Index aktuelle Linie */
  int		 im_rechteck	= 0;	/* Anzahl Linien im Rechteck */

  /*	Zeiger auf Feldanfang setzen
  */
  pl = linien;

  /*	Feld durchmustern, dabei fuer jede Linie pruefen, ob
	sie im Rechteck liegt.
	Falls ja, dann Ergebniszaehler erhoehen.
  */
  for(i = 0; i < anzahl_linien; i++) {
    if(linie_im_rechteck(pl, &test_re)) {
      im_rechteck++;
    }
    pl++;
  }

  /*	Ergebnis ausgeben
  */
  printf("%d Linien liegen vollstaendig im Rechteck.\n", im_rechteck);

  /*    SVG-Graphik ausgeben
  */
  svg_ausgabe();

  return 0;
}



/* +++++ ab hier SVG-Ausgabe, nicht bearbeiten +++++ */


/** Style-Indizes
*/

enum {
  ST_GNL        = 0,    /**< Gitternetz-Linien. */
  ST_RAND ,             /**< Umrandungs-Rechteck. */
  ST_TESTRE ,           /**< Test-Rechteck. */
  ST_LINIE_DRIN ,       /**< Linie im Rechteck. */
  ST_LINIE_DRAUSSEN ,   /**< Linie nicht vollstaendig im Rechteck. */
  ST_TEXT_X ,           /**< Beschriftung x-Achse. */
  ST_TEXT_Y ,           /**< Beschriftung y-Achse. */

  ST_ANZAHL             /**< Anzahl der Styles. */
};


static
void
svg_ausgabe(void)
{
  char       buf[64];
  he_svg_t  *svg    = NULL;
  size_t     lino   = 0;
  int        i      = 0;

  /*    SVG-Datei oeffnen.
  */
  svg = hsm_et_svg_open(
    dn_erg, 800L, 600L, HE_SVG_UNIT_PX, -6.0, 15.0, -2.0, 15.0, ST_ANZAHL, NULL
  );
  if (NULL == svg) {
    /* FEHLER: SVG nicht geoeffnet, Fehlermeldung von Funktion ausgegeben */
    goto ende;
  }
 
  /*    CSS-Klassen fuer Styling benutzen.
  */
  hsm_et_svg_set_use_css(svg, 1, NULL);

  /*    Styles einrichten.
  */
  /* Gitternetz-Linien */
  hsm_et_svg_style_set_stroke_color_name(
    svg, ST_GNL, HE_SVG_COLOR_LIGHTGRAY, NULL
  );
  hsm_et_svg_style_set_linewidth(svg, ST_GNL, 0.02, NULL);
  /* Umrandungs-Rechteck */
  hsm_et_svg_style_set_stroke_color_name(
    svg, ST_RAND, HE_SVG_COLOR_DARKGRAY, NULL
  );
  hsm_et_svg_style_set_fill_color_name(svg, ST_RAND, HE_SVG_COLOR_NONE, NULL);
  hsm_et_svg_style_set_linewidth(svg, ST_RAND, 0.03, NULL);
  /* Test-Rechteck */
  hsm_et_svg_style_set_stroke_color_name(
    svg, ST_TESTRE, HE_SVG_COLOR_BLACK, NULL
  );
  hsm_et_svg_style_set_fill_color_name(svg, ST_TESTRE,HE_SVG_COLOR_NONE, NULL);
  hsm_et_svg_style_set_linewidth(svg, ST_TESTRE, 0.05, NULL);
  /* Linien im Rechteck */
  hsm_et_svg_style_set_stroke_color_name(
    svg, ST_LINIE_DRIN, HE_SVG_COLOR_GREEN, NULL
  );
  hsm_et_svg_style_set_linewidth(svg, ST_LINIE_DRIN, 0.05, NULL);
  /* Linien nicht vollstaendig im Rechteck */
  hsm_et_svg_style_set_stroke_color_name(
    svg, ST_LINIE_DRAUSSEN, HE_SVG_COLOR_RED, NULL
  );
  hsm_et_svg_style_set_linewidth(svg, ST_LINIE_DRAUSSEN, 0.05, NULL);
  /* Text x-Achse */
#if 0
  hsm_et_svg_style_set_fill_color_name(
    svg, ST_TEXT_X, HE_SVG_COLOR_BLACK, NULL
  );
#endif
  hsm_et_svg_style_set_font_family_name(
    svg, ST_TEXT_X,
    "\"Calibri\",\"Tahoma\",\"Verdana\",\"Arial\",sans-serif",
    NULL
  );
  hsm_et_svg_style_set_font_size(svg, ST_TEXT_X, 0.5, NULL);
  hsm_et_svg_style_set_horizontal_align(
    svg, ST_TEXT_X, HE_SVG_ALIGN_H_CENTERED, NULL
  );
  hsm_et_svg_style_set_vertical_align(
    svg, ST_TEXT_X, HE_SVG_ALIGN_V_TOP, NULL
  );
  /* Text y-Achse */
#if 0
  hsm_et_svg_style_set_fill_color_name(
    svg, ST_TEXT_Y, HE_SVG_COLOR_BLACK, NULL
  );
#endif
  hsm_et_svg_style_set_font_family_name(
    svg, ST_TEXT_Y,
    "\"Calibri\",\"Tahoma\",\"Verdana\",\"Arial\",sans-serif",
    NULL
  );
  hsm_et_svg_style_set_font_size(svg, ST_TEXT_Y, 0.5, NULL);
  hsm_et_svg_style_set_horizontal_align(
    svg, ST_TEXT_Y, HE_SVG_ALIGN_H_RIGHT, NULL
  );
  hsm_et_svg_style_set_vertical_align(
    svg, ST_TEXT_Y, HE_SVG_ALIGN_V_MIDDLE, NULL
  );

  /*    Gitternetz-Linien zeichnen.
  */
  for (i = -3; i < 14; i++) {
    hsm_et_svg_draw_line(
      svg,
      ((0 == i) ? (ST_RAND) : (ST_GNL)),
      (double)i, -1.0, (double)i, 14.0, NULL
    );
  }
  for (i = 0; i < 14; i++) {
    hsm_et_svg_draw_line(
      svg,
      ((0 == i) ? (ST_RAND) : (ST_GNL)),
      -4.0, (double)i, 14.0, (double)i, NULL
    );
  }

  /*    Umrandung zeichnen.
  */
  hsm_et_svg_draw_rectangle(
    svg, ST_RAND, -4.0, -1.0, 14.0, 14.0, NULL
  );

  /*    Beschriftung x-Achse.
  */
  for (i = -4; i < 15; i++) {
    sprintf(buf, "%d", i);
    hsm_et_svg_draw_text(
      svg, ST_TEXT_X, (double)i, -1.75, 0.0, buf, 0, NULL
    );
  }

  /*    Beschriftung y-Achse.
  */
  for (i = -1; i < 15; i++) {
    sprintf(buf, "%d", i);
    hsm_et_svg_draw_text(
      svg, ST_TEXT_Y, -4.25, ((double)i - 0.2), 0.0, buf, 0, NULL
    );
  }

  /*    Test-Rechteck.
  */
  hsm_et_svg_draw_rectangle(
    svg, ST_TESTRE, test_re.xmin, test_re.ymin, test_re.xmax, test_re.ymax,
    NULL
  );

  /*    Linien.
  */
  for (lino = 0; lino < anzahl_linien; lino++) {
    hsm_et_svg_draw_line(
      svg,
      (
        (0 != linie_im_rechteck(&(linien[lino]), &test_re))
        ? (ST_LINIE_DRIN)
        : (ST_LINIE_DRAUSSEN)
      ), 
      linien[lino].P0.x, linien[lino].P0.y,
      linien[lino].P1.x, linien[lino].P1.y,
      NULL
    );
  }

  ende:
  /*    SVG-Datei schliessen.
  */
  if (NULL != svg) {
    hsm_et_svg_close(svg, NULL);
  }
}


/* vim: set ai sw=4 ts=4 expandtab : */
